import re
from django.conf import settings
from django.contrib.auth.decorators import login_required
from privateviews.decorators import PublicView


class LoginRequiredMiddleware(object):

    def __init__(self):
        self.public_patterns = []
        self.public_views = []

        if hasattr(settings, 'PUBLIC_VIEWS'):
            for view_path in settings.PUBLIC_VIEWS:
                view = self.get_view(view_path)
                self.public_views.append(view)
        if hasattr(settings, 'PUBLIC_PATHS'):
            for public_path in settings.PUBLIC_PATHS:
                self.public_patterns.append(re.compile(public_path))
        if hasattr(settings, 'LOGIN_URL'):
            pattern = re.compile(r'^%s$' % re.escape(settings.LOGIN_URL))
            self.public_patterns.append(pattern)

    def get_view(self, view_path):
        i = view_path.rfind('.')
        module_path, view_name = view_path[:i], view_path[i + 1:]
        module = __import__(module_path, globals(), locals(), [view_name])
        return getattr(module, view_name)

    def matches_public_view(self, view):
        if self.public_views:
            for public_view in self.public_views:
                if view == public_view:
                    return True
        return False

    def matches_public_path(self, path):
        if self.public_patterns:
            for pattern in self.public_patterns:
                if pattern.match(path) is not None:
                    return True
        return False

    def process_view(self, request, view_func, view_args, view_kwargs):
        if (request.user.is_authenticated()
            or isinstance(view_func, PublicView)
            or self.matches_public_path(request.path)
            or self.matches_public_view(view_func)):
            return None
        else:
            return login_required(view_func)(request, *view_args, **view_kwargs)



from functools import update_wrapper


def login_not_required(view_func):
    """
    Decorator which marks the given view as public (no login required).
    """
    return PublicView(view_func)


class PublicView(object):
    """
    Forces a view to be public (no login required).
    """
    def __init__(self, view_func):
        self.view_func = view_func
        update_wrapper(self, view_func)

    def __get__(self, obj, cls=None):
        view_func = self.view_func.__get__(obj, cls)
        return PublicView(view_func)

    def __call__(self, request, *args, **kwargs):
        return self.view_func(request, *args, **kwargs)



from django.test import TestCase
from django.test.client import RequestFactory


class PrivateViewsTestCase(TestCase):

    def setUp(self):
        self.factory = RequestFactory()

    def test_undecorated_view(self):
        response = self.client.get('/undecorated/')
        self.assertEqual(response.status_code, 302)
        self.assertTrue('accounts/login' in response['Location'])

    def test_decorated_view(self):
        response = self.client.get('/test-decorator/')
        self.assertEqual(response.status_code, 200)

    def test_undecorated_class_based_view(self):
        response = self.client.get('/test-undecorated-class-based-view/')
        self.assertEqual(response.status_code, 302)
        self.assertTrue('accounts/login' in response['Location'])

    def test_decorated_class_based_view(self):
        response = self.client.get('/test-decorated-class-based-view/')
        self.assertEqual(response.status_code, 200)

    def test_list_public_views(self):
        """View listed in settings.PUBLIC_VIEWS"""
        response = self.client.get('/test-public-views/')
        self.assertEqual(response.status_code, 200)

    def test_list_public_paths(self):
        """View listed in settings.PUBLIC_PATHS"""
        response = self.client.get('/test-public-paths/')
        self.assertEqual(response.status_code, 200)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
from distutils.core import setup
import privateviews

version = privateviews.__version__

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()

setup(
    name='django-private-views',
    version=version,
    description='Site-wide login protection.',
    author='Julien Phalip',
    url='https://github.com/dabapps/django-private-views',
    packages=['privateviews', ],
    install_requires=[],
    license='BSD',
)



from django.http import Http404


def private_404(request):
    """
    A catch-all 404 page.

    Using LoginRequiredMiddleware together with this view will ensure that
    users need to be logged in to see 404 responses.
    """
    raise Http404()
